
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.Drawing.Text

Public Class AnalogMeter
    Inherits PictureBox

    Friend TimeString As String = "00:00:00"
    Friend TaValue As Single
    Friend Value As Single
    Friend RingingCounter As Int32
    Friend RingingType As Int32

    Private textFont1 As Font = New Font("Arial", 11, FontStyle.Bold)
    Private textFont2 As Font = New Font("Arial", 15, FontStyle.Bold)
    Private textBrush1 As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))
    Private textBrush2 As Brush = New SolidBrush(Color.FromArgb(200, 50, 0))
    Private indicatorShadowPen As Pen = New Pen(Color.FromArgb(25, 100, 0, 0), 4)
    Private indicatorShadowBluePen As Pen = New Pen(Color.FromArgb(18, 0, 0, 0), 4)
    Private indicatorPen As Pen = New Pen(Color.FromArgb(60, 60, 60), 3)

    Private cursorsBackColor As Color = Color.FromArgb(200, 230, 220, 190)
    Private scaleBackColor As Color = Color.FromArgb(250, 235, 210)

    Public Sub New()
        BackColor = Color.White
        SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        SetStyle(ControlStyles.UserPaint, True)
    End Sub

    Protected Overloads Overrides Sub OnResize(ByVal e As EventArgs)
        MyBase.OnResize(e)
        m_BackgroundChanged = True
    End Sub

    Protected Overloads Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)
        DrawMeter(e.Graphics)
    End Sub

    Private Sub DrawMeter(ByVal g As Graphics)
        If m_BackgroundChanged Then CreateBackground()
        If bgImage Is Nothing Then Return
        g.DrawImageUnscaled(bgImage, 0, 0)
        g.SmoothingMode = SmoothingMode.HighQuality
        DrawLine(Value, g)
        g.FillRectangle(New SolidBrush(cursorsBackColor), 0, 182, Width, Height - 182)
    End Sub


    ' ==========================================================================================================
    '  BACKGROUND BITMAP
    ' ==========================================================================================================
    Private bgImage As Bitmap = Nothing
    Private m_BackgroundChanged As Boolean = False

    Private Sub CreateBackground()
        If Width < 1 OrElse Height < 1 Then Return
        Dim bmp As New Bitmap(Width, Height)
        Dim g As Graphics = Graphics.FromImage(bmp)
        g.SmoothingMode = SmoothingMode.HighQuality
        g.InterpolationMode = InterpolationMode.HighQualityBicubic
        g.TextRenderingHint = TextRenderingHint.AntiAliasGridFit
        g.Dispose()
        ' ----------------------------------------------------------------- set the background image
        If bgImage IsNot Nothing Then bgImage.Dispose()
        bgImage = bmp
        ' -----------------------------------------------------------------
        m_BackgroundChanged = False
        Me.Invalidate()
    End Sub


    ' ==========================================================================================================
    '  DRAW SCALE LINES
    ' ==========================================================================================================
    Private Const Krad As Single = CSng(Math.PI / 180)
    Private Shared angle1 As Single = 0
    Private Shared angle2 As Single = 0

    Private Sub DrawLine(ByVal value As Single, _
                         ByVal g As Graphics)

        Dim rtype As Int32 = RingingType

        If RingingCounter > 2 Then
            RingingCounter -= 1
            If RingingCounter <= 2 Then RingingCounter = 0
            rtype = 0
        End If

        Select Case rtype
            Case 0
                ' ------------------------------------------------------- damped no ringing
                SmoothValue(angle1, value, 0.4)
                SmoothValue(angle2, angle1, 0.4)
                value = angle2
                If value >= 74 Then
                    If RecordingSession Then NewMarker = 11
                End If
                If value <= -44 Then
                    If RecordingSession Then NewMarker = 10
                End If

            Case 1
                ' ------------------------------------------------------- ringing slow
                If value > 72 Then value = 72
                If value < -42 Then value = -42
                angle1 -= (angle2 - angle1) * 0.8F
                SmoothValue(angle1, value, 0.3)
                SmoothValue(angle2, angle1, 0.3)
                value = angle2
                If value > 75 Then
                    value = 75
                    angle1 = 60
                    angle2 = 75
                    PlayClickSound()
                    RingingCounter += 1
                    If RecordingSession Then NewMarker = 11
                    'Form1.SetNewMarker(10) ' to add markers to an audio track while playing it
                End If
                If value < -45 Then
                    value = -45
                    angle1 = -30
                    angle2 = -45
                    PlayClickSound()
                    RingingCounter += 1
                    If RecordingSession Then NewMarker = 10
                    'Form1.SetNewMarker(10) ' to add markers to an audio track while playing it
                End If

                'Case 2
                '    ' ------------------------------------------------------- ringing fast (unused)
                '    If value > 72 Then value = 72
                '    If value < -42 Then value = -42
                '    angle1 -= (angle2 - angle1) * 1.3F
                '    SmoothValue(angle1, value, 0.4)
                '    SmoothValue(angle2, angle1, 0.4)
                '    value = angle2
                '    If value > 75 Then
                '        value = 75
                '        angle1 = 65
                '        angle2 = 75
                '        PlayClickSound()
                '        RingingCounter += 1
                '        If RecordingSession Then NewMarker = 11
                '        'Form1.SetNewMarker(10) ' to add markers to an audio track while playing it
                '    End If
                '    If value < -45 Then
                '        value = -45
                '        angle1 = -35
                '        angle2 = -45
                '        PlayClickSound()
                '        RingingCounter += 1
                '        If RecordingSession Then NewMarker = 10
                '        'Form1.SetNewMarker(10) ' to add markers to an audio track while playing it
                '    End If

        End Select

        Dim angle As Single = value + 76

        ' Value is -45 to 75
        ' Angle is  31 to 151

        ' ----------------------------------------------------------------------- CLICK HERE MESSAGE
        If Not PlayingAudio Then
            'If angle < 41 Or angle > 139 Then
            '    g.DrawString("Click here to SET", textFont2, textBrush2, 120, 130)
            'End If

            Dim v As Int32 = CInt(Math.Abs(value))
            If v > 40 Then v = 40
            Dim r1 As Int32 = CInt(scaleBackColor.R - v * 2)
            Dim g1 As Int32 = CInt(scaleBackColor.G - v * 2)
            Dim b1 As Int32 = CInt(scaleBackColor.B - v * 2)
            If r1 < 0 Then r1 = 0
            If g1 < 0 Then g1 = 0
            If b1 < 0 Then b1 = 0
            Dim br As Brush = New SolidBrush(Color.FromArgb(r1, g1, b1))
            g.DrawString("Click here to SET", textFont2, br, 120, 130)
        End If

        ' ------------------------------------------------------------------------ INDICATOR
        Dim p(3) As PointF
        p(0) = New PointF(Width / 2.0F - 5, 280)

        Dim cosa As Double = Math.Cos(angle * Krad)
        Dim sina As Double = Math.Sin(angle * Krad)
        p(1) = New PointF(CSng(p(0).X - 213 * cosa), CSng(p(0).Y - 213 * sina))

        cosa = Math.Cos((angle - 180 + 20) * Krad)
        sina = Math.Sin((angle - 180 + 20) * Krad)
        p(2) = New PointF(CSng(p(1).X - 12 * cosa), CSng(p(1).Y - 12 * sina))

        cosa = Math.Cos((angle - 180 - 20) * Krad)
        sina = Math.Sin((angle - 180 - 20) * Krad)
        p(3) = New PointF(CSng(p(1).X - 12 * cosa), CSng(p(1).Y - 12 * sina))

        Dim dx As Single = 4
        Dim dy As Single = 2
        g.DrawLine(indicatorShadowPen, p(0).X + dx, p(0).Y + dy, p(1).X + dx, p(1).Y + dy)
        g.DrawLine(indicatorShadowPen, p(1).X + dx, p(1).Y + dy, p(2).X + dx, p(2).Y + dy)
        g.DrawLine(indicatorShadowPen, p(1).X + dx, p(1).Y + dy, p(3).X + dx, p(3).Y + dy)

        dx = 8
        dy = 2
        g.DrawLine(indicatorShadowBluePen, p(0).X + dx, p(0).Y + dy, p(1).X + dx, p(1).Y + dy)
        g.DrawLine(indicatorShadowBluePen, p(1).X + dx, p(1).Y + dy, p(2).X + dx, p(2).Y + dy)
        g.DrawLine(indicatorShadowBluePen, p(1).X + dx, p(1).Y + dy, p(3).X + dx, p(3).Y + dy)

        g.DrawLine(indicatorPen, p(0), p(1))
        g.DrawLine(indicatorPen, p(1), p(2))
        g.DrawLine(indicatorPen, p(1), p(3))

        ' ------------------------------------------------------------------------ TEXT
        g.DrawString("Total time", textFont1, textBrush1, 3, 4)
        g.DrawString(TimeString, textFont2, textBrush1, 3, 20)

        g.DrawString("TA", textFont1, textBrush1, 394, 4)
        g.DrawString(TaValue.ToString("0.00", GCI), textFont2, textBrush1, 376, 20)

        If PlayingAudio Then
            g.FillRectangle(Brushes.LightYellow, 128, 3, 184, 20)
            g.DrawString("Playing recorded session", textFont1, textBrush2, 130, 4)
        End If

    End Sub

End Class
