﻿Public Class DataArray

    Friend Structure DataSample
        Dim TaValue As Single
        Dim SignalValue As Single
        Dim Marker As Byte
        Dim AudioLevel As Byte
        Sub New(ByVal _TaValue As Single, _
                ByVal _SignalValue As Single, _
                ByVal _Marker As Byte, _
                ByVal _AudioLevel As Byte)
            TaValue = _TaValue
            SignalValue = _SignalValue
            Marker = _Marker
            AudioLevel = _AudioLevel
        End Sub
    End Structure

    Private m_ArrayMaxLength As Int32
    Private m_WriteIndex As Int32
    Private m_DataArray() As DataSample
    Private m_NumEntries As Int32

    Friend Sub New(ByVal ArrayMaxLength As Int32)
        m_ArrayMaxLength = ArrayMaxLength
        ReDim m_DataArray(m_ArrayMaxLength - 1)
        Flush()
    End Sub

    Friend Sub Flush()
        m_WriteIndex = 0
        m_NumEntries = 0
    End Sub

    Friend Sub Fill(ByVal TaValue As Single, _
                    ByVal SignalValue As Single, _
                    ByVal Marker As Byte, _
                    ByVal AudioLevel As Byte)
        Dim ds As DataSample = New DataSample(TaValue, SignalValue, Marker, AudioLevel)
        For i As Int32 = 0 To m_ArrayMaxLength - 1
            m_DataArray(i) = ds
        Next
    End Sub

    ' This function uses 200 uS to scan a 60-minute data array
    Friend Function CalcTaCounterFromStart(ByVal index As Int32) As Single
        Dim counter As Single = 0
        Dim ta As Single = 0
        Dim oldta As Single = m_DataArray(0).TaValue
        For i As Int32 = 1 To index - 1
            ta = m_DataArray(i).TaValue
            If ta < oldta Then counter += oldta - ta
            oldta = ta
        Next
        Return counter
    End Function

    Friend Sub DeleteAllMarkers()
        For i As Int32 = 0 To DataArray1.GetNumEntries() - 1
            m_DataArray(i).Marker = 0
        Next
    End Sub

    Friend Sub FillUnusedArea(ByVal TaValue As Single, _
                              ByVal SignalValue As Single, _
                              ByVal Marker As Byte, _
                              ByVal AudioLevel As Byte)
        Dim ds As DataSample = New DataSample(TaValue, SignalValue, Marker, AudioLevel)
        For i As Int32 = m_WriteIndex To m_ArrayMaxLength - 1
            m_DataArray(i) = ds
        Next
    End Sub

    Friend Function GetWriteIndex() As Int32
        Return m_WriteIndex
    End Function

    Friend Sub SetWriteIndex(ByVal WriteIndex As Int32)
        If WriteIndex < 0 Then WriteIndex = 0
        If WriteIndex > m_ArrayMaxLength - 1 Then WriteIndex = m_ArrayMaxLength - 1
        m_WriteIndex = WriteIndex
    End Sub

    Friend Sub SetNumEntries(ByVal n As Int32)
        If n < 0 Then n = 0
        If n > m_ArrayMaxLength - 1 Then n = m_ArrayMaxLength - 1
        m_NumEntries = n
    End Sub

    Friend Function GetNumEntries() As Int32
        Return m_NumEntries
    End Function

    Friend Function GetBuffer() As DataSample()
        Return m_DataArray
    End Function

    Friend Sub AddElement(ByVal TaValue As Single, _
                          ByVal SignalValue As Single, _
                          ByVal Marker As Byte, _
                          ByVal AudioLevel As Byte)
        Dim ds As DataSample
        ds = New DataSample(TaValue, SignalValue, Marker, AudioLevel)
        m_DataArray(m_WriteIndex) = ds
        m_WriteIndex += 1
        If m_WriteIndex > m_ArrayMaxLength - 1 Then m_WriteIndex = m_ArrayMaxLength - 1
        m_NumEntries += 1
        If m_NumEntries > m_ArrayMaxLength Then m_NumEntries = m_ArrayMaxLength
    End Sub

    Friend Sub SetMarker(ByVal index As Int32, ByVal MarkerValue As Byte)
        If index < 0 Then index = 0
        If index > m_ArrayMaxLength - 1 Then index = m_ArrayMaxLength - 1
        m_DataArray(index).Marker = MarkerValue
    End Sub

    Friend Sub SetAudioLevel(ByVal index As Int32, ByVal AudioLevel As Byte)
        If index < 0 Then index = 0
        If index > m_ArrayMaxLength - 1 Then index = m_ArrayMaxLength - 1
        m_DataArray(index).AudioLevel = AudioLevel
    End Sub

    Friend Sub SetElement(ByVal index As Int32, ByVal Value As DataSample)
        If index < 0 Then index = 0
        If index > m_ArrayMaxLength - 1 Then index = m_ArrayMaxLength - 1
        m_DataArray(index) = Value
    End Sub

    Friend Function GetElement(ByVal Index As Int32) As DataSample
        If Index < 0 Then Index = 0
        If Index > m_ArrayMaxLength - 1 Then Index = m_ArrayMaxLength - 1
        Return m_DataArray(Index)
    End Function

End Class
