﻿Imports System.Drawing.Drawing2D

Friend Class VuMeter

    Private f As Font
    Private rec As Rectangle
    Private stringFormat As New StringFormat()
    Private w As Int32
    Private h As Int32
    Private b1 As Drawing2D.LinearGradientBrush
    Private b2 As Brush
    Private db As Double
    Private g As Graphics
    Private Color1 As Color = Color.FromArgb(255, 0, 0)
    Private Color2 As Color = Color.FromArgb(255, 200, 0)
    Private Color3 As Color = Color.FromArgb(100, 255, 0)

    Friend Sub Draw(ByVal pbox As PictureBox, _
                    ByVal value As Double, _
                    Optional ByVal MarkPoint As Double = 0, _
                    Optional ByVal LogScale As Boolean = False, _
                    Optional ByVal LogScaleMinDb As Int32 = -40, _
                    Optional ByVal ColorType As Int32 = 0, _
                    Optional ByVal Percentual As Boolean = False, _
                    Optional ByVal Text As String = "", _
                    Optional ByVal TextSize As Single = 0.4, _
                    Optional ByVal PercentualSize As Single = 0.85)
        '
        ' ----------------------------------------------------------- 
        If Not pbox.Visible Then Exit Sub
        If pbox.ClientSize.Width = 0 OrElse pbox.ClientSize.Height = 0 Then Exit Sub
        ' ----------------------------------------------------------- 
        If pbox.Image Is Nothing OrElse pbox.ClientSize.Width <> w OrElse pbox.ClientSize.Height <> h Then
            w = pbox.ClientSize.Width
            h = pbox.ClientSize.Height
            pbox.Image = New Bitmap(w, h)
            g = Graphics.FromImage(pbox.Image)
            b1 = CreateFillBrush(pbox, _
                                 If(w > h, LinearGradientMode.Horizontal, LinearGradientMode.Vertical), _
                                 ColorType)
            b2 = New SolidBrush(pbox.BackColor)
        End If
        ' --------------------------------------------------------------- calc Decibels and LogScale 
        db = VoltToDecibel(value)
        If LogScale Then value = CSng(1 - db / LogScaleMinDb)
        If value < 0 Then value = 0
        If value > 1 Then value = 1
        ' --------------------------------------------------------------- 
        If w > h Then
            Dim v As Int32 = CInt(value * w)
            g.FillRectangle(b1, 0, 0, v, h)
            g.FillRectangle(b2, v, 0, w, h)
            ' ----------------------------------------------------------- mark line
            If MarkPoint > 0 Then
                v = CInt(MarkPoint * pbox.Width)
                g.DrawLine(Pens.Black, v, 0, v, h)
            End If
        Else
            Dim v As Int32 = CInt(value * h)
            g.FillRectangle(b1, 0, h - v, w, h)
            g.FillRectangle(b2, 0, 0, w, h - v)
            ' ----------------------------------------------------------- mark line
            If MarkPoint > 0 Then
                v = CInt(MarkPoint * pbox.Height)
                g.DrawLine(Pens.Black, 0, h - v, w, h - v)
            End If
        End If

        Dim str As String = ""
        ' --------------------------------------------------------------- prepare Percentual or dB string
        If Percentual Then
            str = CInt(value * 100).ToString + "%"
        Else
            If db > -60 Then
                str = db.ToString("0") & " dB"
            End If
        End If
        ' --------------------------------------------------------------- print Percentual or dB
        f = New Font("Tahoma", CInt(h * PercentualSize), FontStyle.Regular, GraphicsUnit.Pixel)
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
        stringFormat.Alignment = StringAlignment.Center
        stringFormat.LineAlignment = StringAlignment.Center
        rec = New Rectangle(1, 1, w, h)
        g.DrawString(str, f, Brushes.Black, rec, stringFormat)
        ' --------------------------------------------------------------- print text
        If Text <> "" Then
            f = New Font("Tahoma", CInt(h * TextSize), FontStyle.Bold)
            stringFormat.Alignment = StringAlignment.Near
            stringFormat.LineAlignment = StringAlignment.Center
            rec = New Rectangle(1, 1, w, h)
            g.DrawString(Text, f, Brushes.Black, rec, stringFormat)
        End If
        ' --------------------------------------------------------------- final refresh - no flickering
        pbox.Refresh()
    End Sub

    Private Function CreateFillBrush(ByVal pbox As PictureBox, _
                                     ByVal direction As LinearGradientMode, _
                                     ByVal ColorType As Int32) As LinearGradientBrush
        '
        CreateFillBrush = New LinearGradientBrush(pbox.ClientRectangle, _
                                                  Color.Black, _
                                                  Color.Black, _
                                                  direction)
        Dim myBlend As ColorBlend = New ColorBlend()
        Select Case ColorType
            Case 0
                If direction = LinearGradientMode.Vertical Then
                    myBlend.Colors = New Color() {Color1, Color2, Color3}
                Else
                    myBlend.Colors = New Color() {Color3, Color2, Color1}
                End If
                myBlend.Positions = New Single() {0.0F, 0.5F, 1.0F}
            Case 1
                myBlend.Colors = New Color() {Color3, Color3, Color3}
                myBlend.Positions = New Single() {0.0F, 0.5F, 1.0F}
        End Select
        CreateFillBrush.InterpolationColors = myBlend
    End Function

    Private Function VoltToDecibel(ByRef v As Double) As Double
        VoltToDecibel = 20 * Math.Log(v, 10)
    End Function

End Class

