﻿Imports System.Drawing.Imaging

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    Friend Form2_VisibleAtStart As Boolean

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        If s(2) <> "0" Then
            Return Title & " - V" & s(0) & "." & s(1) & "." & s(2)
        Else
            Return Title & " - V" & s(0) & "." & s(1)
        End If
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function
    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub SaveConfigurationAs()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        sfd.FileName = AssemblyName() & "_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadConfiguration()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
        End If
    End Sub

    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        If iniFileName = "" Then
            iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
        End If
        '
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            '
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            'f.WriteLine(TabString("Form1_Width", r.Width))
            'f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine("")
            f.WriteLine(" Controls")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("ControlsType", Form1.ControlsType))
            f.WriteLine(TabString("Fast", Form1.chk_Fast.Checked))
            f.WriteLine(TabString("SetTime", Form1.tkbar_SetTime.Value))
            f.WriteLine(TabString("Gain", Form1.tkbar_Gain.Value))
            f.WriteLine(TabString("Speed", Form1.tkbar_Speed.Value))
            f.WriteLine(TabString("DeltaTime", DeltaTime))
            '
            f.WriteLine("")
            f.WriteLine(" InOut Slots")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("SlotIn", Form1.txt_SlotIn.Text))
            f.WriteLine(TabString("SlotOut", Form1.txt_SlotOut.Text))
            f.WriteLine(TabString("SlotSet", Form1.txt_SlotSet.Text))
            f.WriteLine(TabString("SlotSetTime", Form1.txt_SlotSetTime.Text))
            f.WriteLine(TabString("SlotGain", Form1.txt_SlotGain.Text))
            f.WriteLine(TabString("SlotSpeed", Form1.txt_SlotSpeed.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Audio")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("RecordAudio", Form1.chk_RecordAudio.Checked))
            f.WriteLine(TabString("SelectedAudioIn", Form1.SelectedAudioIn))
            f.WriteLine(TabString("BitsPerSec", Form1.cmb_BitsPerSec.Text))
            '
            f.WriteLine("")
            f.WriteLine(" Last data file")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("LastDataFile", LastDataFile))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = PlatformAdjustedFileName(Application.StartupPath & "\" & AssemblyName() & "_INI.txt")
            LoadAll = True
        End If
        '
        Dim l As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    ' -----------------------------------------------------------------
                    Case "Form1_Top" : If LoadAll Then Form1.Top = CInt(Val(l))
                    Case "Form1_Left" : If LoadAll Then Form1.Left = CInt(Val(l))
                        'Case "Form1_Width" : If LoadAll Then Form1.Width = CInt(Val(l))
                        'Case "Form1_Height" : If LoadAll Then Form1.Height = CInt(Val(l))
                    Case "Form1_WindowState" : If LoadAll Then Form1.WindowState = CType(Val(l), FormWindowState)
                        '
                        ' ----------------------------------------------------------------- controls
                    Case "ControlsType" : Form1.ControlsType = Val_Int(l)
                    Case "Fast" : Form1.chk_Fast.Checked = l = "True"
                    Case "SetTime" : Form1.tkbar_SetTime.Value = Val_Int(l)
                    Case "Gain" : Form1.tkbar_Gain.Value = Val_Int(l)
                    Case "Speed" : Form1.tkbar_Speed.Value = Val_Int(l)
                    Case "DeltaTime" : DeltaTime = Val_Single(l)
                        ' ----------------------------------------------------------------- slots
                    Case "SlotIn" : Form1.txt_SlotIn.Text = l
                    Case "SlotOut" : Form1.txt_SlotOut.Text = l
                    Case "SlotSet" : Form1.txt_SlotSet.Text = l
                    Case "SlotSetTime" : Form1.txt_SlotSetTime.Text = l
                    Case "SlotGain" : Form1.txt_SlotGain.Text = l
                    Case "SlotSpeed" : Form1.txt_SlotSpeed.Text = l
                        ' ----------------------------------------------------------------- audio
                    Case "RecordAudio" : Form1.chk_RecordAudio.Checked = l = "True"
                    Case "SelectedAudioIn" : Form1.SelectedAudioIn = Val_Int(l)
                    Case "BitsPerSec" : Form1.cmb_BitsPerSec.Text = l
                        ' ----------------------------------------------------------------- last data file
                    Case "LastDataFile" : LastDataFile = l
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub


    ' ==================================================================================================
    '  SAVE IMAGE
    ' ==================================================================================================
    Public Sub SaveImage(ByVal img As Image, _
                         ByVal filename As String, _
                         ByVal extension As String, _
                         ByVal Quality As Int32)

        extension = LCase(extension)
        filename = RemoveExtension(filename)
        filename += "." & extension
        '
        If img Is Nothing Then Exit Sub
        Try
            File_Kill(filename)
            If extension = "jpg" Then
                Dim ImageEncoders() As ImageCodecInfo = ImageCodecInfo.GetImageEncoders()
                Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality
                Dim myEncoderParameters As New EncoderParameters(1)
                Dim myEncoderParameter As New EncoderParameter(myEncoder, 100)
                myEncoderParameters.Param(0) = myEncoderParameter
                img.Save(filename, ImageEncoders(1), myEncoderParameters)
            Else
                img.Save(filename, ImageFormatFromFileExtension(extension))
            End If
        Catch
            MsgBox("Image save error", MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Function ImageFormatFromFileExtension(ByVal extension As String) As ImageFormat
        Select Case LCase(extension)
            Case "jpg" : Return ImageFormat.Jpeg
            Case "png" : Return ImageFormat.Png
            Case "tiff" : Return ImageFormat.Tiff
            Case "exif" : Return ImageFormat.Exif
            Case "emf" : Return ImageFormat.Emf
            Case "wmf" : Return ImageFormat.Wmf
            Case "gif" : Return ImageFormat.Gif
            Case "bmp" : Return ImageFormat.Bmp
                'Case "ico" : Return ImageFormat.Icon
            Case Else : Return ImageFormat.Jpeg
        End Select
    End Function

    Friend Sub File_Kill(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            My.Computer.FileSystem.DeleteFile(filename, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)
        End If
    End Sub

    Friend Sub SaveImage(ByVal Prefix As String, ByVal cnt As Control)
        Dim img As Image = GetImage(cnt)
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.Filter = "Portable network graphics (*.png)|*png|JPEG image (*.jpg)|*.jpg"
        sfd.FileName = Prefix & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveImage(img, sfd.FileName, If(sfd.FilterIndex = 2, "jpg", "png"), 100)
        End If
    End Sub
    Friend Function GetImage(ByVal cnt As Control) As Bitmap
        Dim s As Size = cnt.Size
        Dim bmp As Bitmap = New Bitmap(s.Width, s.Height)
        cnt.DrawToBitmap(bmp, New Rectangle(0, 0, s.Width, s.Height))
        Return bmp
    End Function


    ' =============================================================
    '  SAVE DATA
    ' =============================================================
    Friend Sub SaveData()
        If LastDataFile = "" Then Return
        Form1.StopSession()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.FileName = IO.Path.GetFileName(LastDataFile)
        If My.Computer.FileSystem.FileExists(LastDataFile) Then
            sfd.InitialDirectory = IO.Path.GetDirectoryName(LastDataFile)
            'Sendkeys to select the LastDataFile (works only if the FileDialog is in List mode)
            'SendKeys.Send("+{TAB}+{TAB}" + IO.Path.GetFileName(LastDataFile) + "{TAB}{TAB}")
        Else
            sfd.InitialDirectory = Application.StartupPath + "\DataFiles\"
            IO.Directory.CreateDirectory(sfd.InitialDirectory)
            sfd.FileName = "Session_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        End If
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Data file (*.txt)|*.txt"
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Form1.ChangeTempFilesToFinalName(sfd.FileName)
            SaveDataTo(sfd.FileName)
        Else
            Form1.DeleteTempFiles()
        End If
    End Sub

    Friend Sub AskForSaveData()
        If MessageBox.Show("Session data not saved." + _
                           vbCrLf + vbCrLf + "Save session now?", _
                           "Warning from Emotion Meter", _
                           MessageBoxButtons.YesNo, _
                           MessageBoxIcon.Warning, _
                           MessageBoxDefaultButton.Button2, _
                           CType(&H40000, MessageBoxOptions)) = DialogResult.Yes Then
            Form1.StopSession()
            Form1.PlayAudioStop()
            SaveData()
        End If
        Form1.DeleteTempFiles()
        UnsavedData = False
    End Sub

    Friend Sub SaveDataTo(ByVal filename As String)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        '
        UnsavedData = False
        LastDataFile = filename
        StatusStrip_ShowFileName(LastDataFile)
        '
        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(filename))
        Dim Buffer() As DataArray.DataSample
        Dim usedLen As Int32
        Dim totalLen As Int32
        '
        Using sw As IO.StreamWriter = New IO.StreamWriter(filename)
            '
            Buffer = DataArray1.GetBuffer
            totalLen = Buffer.Length
            usedLen = DataArray1.GetNumEntries
            sw.WriteLine("------------------------------ TA_BUFFER")
            For i As Int32 = 0 To usedLen - 1
                sw.WriteLine(Buffer(i).TaValue.ToString(GCI))
            Next
            '
            sw.WriteLine("------------------------------ SIGNAL_BUFFER")
            For i As Int32 = 0 To usedLen - 1
                sw.WriteLine(Buffer(i).SignalValue.ToString(GCI))
            Next
            '
            sw.WriteLine("------------------------------ MARKERS")
            For i As Int32 = 0 To usedLen - 1
                sw.WriteLine(Buffer(i).Marker.ToString(GCI))
            Next
            '
            sw.WriteLine("------------------------------ AUDIO_LEVELS")
            For i As Int32 = 0 To usedLen - 1
                sw.WriteLine(Buffer(i).AudioLevel.ToString(GCI))
            Next
            '
            sw.WriteLine("------------------------------ RECORD_DATE")
            sw.WriteLine(StatusStrip_GetRecordDate)
        End Using
        '
        'Form1.Text = "Save time: " & sw1.Elapsed.TotalMilliseconds.ToString("0") & " mS"
    End Sub

    ' =============================================================
    '  LOAD DATA
    ' =============================================================
    Friend Sub LoadData()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.Multiselect = True
        ofd.FileName = IO.Path.GetFileName(LastDataFile)
        If My.Computer.FileSystem.FileExists(LastDataFile) Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(LastDataFile)
            'Sendkeys to select the LastDataFile (works only if the FileDialog is in List mode)
            'SendKeys.Send("+{TAB}+{TAB}" + IO.Path.GetFileName(LastDataFile) + "{TAB}{TAB}")
        Else
            ofd.InitialDirectory = Application.StartupPath + "\DataFiles\"
            IO.Directory.CreateDirectory(ofd.InitialDirectory)
        End If
        ofd.Filter = "Data file (*.txt)|*.txt"
        ofd.DefaultExt = ".txt"
        ofd.Multiselect = True
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LoadDataFrom(ofd.FileName)
        End If
    End Sub


    Friend Sub LoadDataFrom(ByVal filename As String)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        '
        If UnsavedData Then AskForSaveData()

        LastDataFile = filename
        StatusStrip_ShowFileName(LastDataFile)
        '
        If Not My.Computer.FileSystem.FileExists(filename) Then
            LastDataFile = ""
            StatusStrip_ShowFileName(LastDataFile)
            Return
        End If
        '
        Form1.Cursor = Cursors.WaitCursor

        Form1.StopSession()
        Form1.PlayAudioStop()
        '
        Dim f As System.IO.StreamReader
        f = IO.File.OpenText(filename)
        Dim i As Int32
        Dim s As String = ""
        Dim buf() As DataArray.DataSample
        '
        DataArray1.Flush()
        DataArray1.Fill(999, -999, 0, 0)
        buf = DataArray1.GetBuffer
        s = f.ReadLine
        '
        If s.Length > 30 AndAlso s.EndsWith("TA_BUFFER") Then
            For i = 0 To buf.Length - 1
                If f.EndOfStream Then Exit For
                s = f.ReadLine
                If s.Length > 30 Then Exit For
                buf(i).TaValue = CSng(Val(s))
            Next
            DataArray1.SetWriteIndex(i)
            DataArray1.SetNumEntries(i)
        End If
        '
        If s.Length > 30 AndAlso s.EndsWith("SIGNAL_BUFFER") Then
            For i = 0 To buf.Length - 1
                If f.EndOfStream Then Exit For
                s = f.ReadLine
                If s.Length > 30 Then Exit For
                buf(i).SignalValue = CSng(Val(s))
            Next
        End If
        '
        If s.Length > 30 AndAlso s.EndsWith("MARKERS") Then
            For i = 0 To buf.Length - 1
                If f.EndOfStream Then Exit For
                s = f.ReadLine
                If s.Length > 30 Then Exit For
                buf(i).Marker = CByte(Val(s))
            Next
        End If
        '
        If s.Length > 30 AndAlso s.EndsWith("AUDIO_LEVELS") Then
            For i = 0 To buf.Length - 1
                If f.EndOfStream Then Exit For
                s = f.ReadLine
                If s.Length > 30 Then Exit For
                buf(i).AudioLevel = CByte(Val(s))
            Next
        End If
        For i = 0 To 99999
            If s.Contains("RECORD_DATE") Then
                If f.EndOfStream Then Exit For
                s = f.ReadLine
                StatusStrip_SetRecordDate(s)
            End If
        Next
        ' 
        f.Close()
        Form1.Cursor = Cursors.Default
        '
        'Form1.Text = "Load time: " & sw1.Elapsed.TotalMilliseconds.ToString("0") & " mS"
    End Sub

End Module
